package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.core.oe.admin.base.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.CourseLevel;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.SelectType;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordIllegallyService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examaudit.ExamAuditInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examaudit.ExamAuditQuery;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ExamRecordIllegallyServiceImpl implements ExamRecordIllegallyService {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordIllegallyServiceImpl.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public Page<ExamAuditInfo> getExamRecordIllegallyList(ExamAuditQuery query, UserDataRule courseRule,
            UserDataRule orgRule) {
        Check.isNull(query, "请求参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试！");

        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize());
        if (courseRule.assertEmptyQueryResult() || orgRule.assertEmptyQueryResult()) {
            return Page.empty(pageable);
        }

        int offset = (query.getPageNo() - 1) * query.getPageSize();
        String countSql = this.queryExamRecordIllegallyListSql(query, true, courseRule, orgRule);
        String querySql = this.queryExamRecordIllegallyListSql(query, false, courseRule, orgRule);
        String pageSql = querySql + " limit " + offset + "," + query.getPageSize();

        if (SelectType.EXPORT == query.getSelectType()) {
            // 仅导出结果
            List<ExamAuditInfo> list = jdbcTemplate.query(querySql, new BeanPropertyRowMapper(ExamAuditInfo.class));
            // 填充其它信息
            this.fillOtherInfo(list, query.getExamId());
            return new PageImpl<>(list, pageable, list.size());
        }

        Long totalElements = jdbcTemplate.queryForObject(countSql, Long.class);
        if (totalElements == null || totalElements == 0) {
            return Page.empty(pageable);
        }

        List<ExamAuditInfo> list = jdbcTemplate.query(pageSql, new BeanPropertyRowMapper(ExamAuditInfo.class));
        if (CollectionUtils.isEmpty(list)) {
            return Page.empty(pageable);
        }

        // 填充其它信息
        this.fillOtherInfo(list, query.getExamId());

        return new PageImpl<>(list, pageable, totalElements);
    }

    private void fillOtherInfo(List<ExamAuditInfo> list, Long examId) {
        ExamSettingsCacheBean exam = CacheHelper.getExamSettings(examId);
        for (ExamAuditInfo info : list) {
            // 考试名称
            info.setExamName(exam.getName());

            // 学习中心名称
            OrgCacheBean org = CacheHelper.getOrg(info.getOrgId());
            info.setOrgName(org.getName());

            // 课程信息
            CourseCacheBean course = CacheHelper.getCourse(info.getCourseId());
            info.setCourseCode(course.getCode());
            info.setCourseName(course.getName());
            info.setCourseLevel(CourseLevel.getCourseLevel(course.getLevel()).getTitle());
        }
    }

    private String queryExamRecordIllegallyListSql(ExamAuditQuery query, boolean isCount, UserDataRule courseRule,
            UserDataRule orgRule) {
        StringBuilder sql = new StringBuilder();
        if (isCount) {
            sql.append(" select count(1)");
        } else {
            sql.append(" select rd.id AS examRecordDataId,rd.exam_id,rd.exam_type,");
            sql.append(" rd.course_id,rd.org_id,rd.exam_stage_id,rd.student_id,");
            sql.append(" rd.exam_student_id,rd.student_code,rd.student_name,rd.identity_number,");
            sql.append(" rd.start_time AS paperStartTime,rd.end_time AS paperSubmitTime,rd.exam_order,");
            sql.append(" rd.face_total_count,rd.face_success_count,rd.face_failed_count,");
            sql.append(" rd.face_success_percent,rd.face_stranger_count,rd.face_verify_result,");
            sql.append(" rd.switch_screen_count,sco.objective_score,a.id,a.status,ill.name AS discipline_type,");
            sql.append(" a.discipline_detail,a.audit_user_name,a.update_time AS paperAuditTime,");
            sql.append(" (select GROUP_CONCAT(DISTINCT pr.source_ip) from ec_oe_exam_process_record pr")
                    .append(" where pr.exam_record_data_id = rd.id) ip");
        }

        sql.append(" from ec_oe_exam_record_data rd ");
        sql.append(" left join ec_oe_exam_audit a on a.exam_record_data_id = rd.id");
        sql.append(" left join ec_oe_illegally_type ill on ill.code = a.discipline_type")
                .append(" and (ill.root_org_id = -1 or ill.root_org_id = rd.root_org_id)");
        sql.append(" left join ec_oe_exam_score sco on sco.exam_record_data_id = rd.id");

        sql.append(" where rd.exam_id = ").append(query.getExamId());
        sql.append(" and rd.exam_record_status != 'EXAM_INVALID'");
        sql.append(" and exists (");
        sql.append(" select aa.exam_record_data_id from ec_oe_exam_audit aa");
        sql.append(" where aa.exam_record_data_id = rd.id and aa.status = 'UN_PASS'");
        if (StringUtils.isNotBlank(query.getDisciplineType())) {
            sql.append(" and aa.discipline_type = '").append(query.getDisciplineType()).append("'");
        }
        sql.append(" )");

        if (courseRule.assertNeedQueryRefIds()) {
            // 限定课程数据权限范围
            if (query.getCourseId() != null) {
                if (courseRule.getRefIds().contains(query.getCourseId())) {
                    sql.append(" and rd.course_id = ").append(query.getCourseId());
                } else {
                    // 不在数据权限范围内，无效查询
                    sql.append(" and rd.course_id = -1");
                }
            } else {
                sql.append(" and rd.course_id in (").append(StringUtils.join(courseRule.getRefIds(), ",")).append(")");
            }
        } else {
            // 未限定数据权限
            if (query.getCourseId() != null) {
                sql.append(" and rd.course_id = ").append(query.getCourseId());
            }
        }

        if (orgRule.assertNeedQueryRefIds()) {
            // 限定机构数据权限范围
            if (query.getOrgId() != null) {
                if (orgRule.getRefIds().contains(query.getOrgId())) {
                    sql.append(" and rd.org_id = ").append(query.getOrgId());
                } else {
                    // 不在数据权限范围内，无效查询
                    sql.append(" and rd.org_id = -1");
                }
            } else {
                sql.append(" and rd.org_id in (").append(StringUtils.join(orgRule.getRefIds(), ",")).append(")");
            }
        } else {
            // 未限定数据权限
            if (query.getOrgId() != null) {
                sql.append(" and rd.org_id = ").append(query.getOrgId());
            }
        }

        if (query.getExamStageId() != null) {
            sql.append(" and rd.exam_stage_id = ").append(query.getExamStageId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and rd.course_level = '").append(query.getCourseLevel()).append("'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and rd.identity_number like '").append(query.getIdentityNumber()).append("%'");
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and rd.student_code like '").append(query.getStudentCode()).append("%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and rd.student_name like '").append(query.getStudentName()).append("%'");
        }

        if (!isCount) {
            sql.append(" order by rd.update_time desc");
        }

        return sql.toString();
    }

}
